/*

kHTTPd -- the next generation

RFC related functions (headers and stuff)

*/

/****************************************************************
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************/


#include <linux/kernel.h>

#include <linux/ctype.h>
#include <linux/errno.h>
#include <linux/malloc.h>
#include <linux/net.h>
#include <linux/sched.h>
#include <linux/skbuff.h>
#include <linux/unistd.h>
#include <linux/file.h>

#include <net/ip.h>
#include <net/sock.h>

#include <asm/atomic.h>
#include <asm/semaphore.h>
#include <asm/processor.h>
#include <asm/uaccess.h>


#include "prototypes.h"
#include "structure.h"
#include "config.h"

atomic_t	MimeCount;

struct MimeType
{
	__u32 	identifier;
	char	type[32-4];  
};

static struct MimeType	MimeTypes[KHTTPD_NUMMIMETYPES];


void AddMimeType(char *Ident,char *Type)
{	
	__u32	*I;
	
	if (strlen(Ident)!=4) 
   	{	
   		printk("httpd: Only 4-byte mime-identifiers are accepted\n");
   		return;
   	}

	if (strlen(Type)>(32-4)) 
   	{	
   		printk("httpd: Mime-string too long.\n");
   		return;
   	}
   	
   	I=(__u32*)Ident;
   	
   	/* FIXME: Need to lock-down all access to the mime-structure here */
   	
   	MimeTypes[atomic_read(&MimeCount)].identifier=*I;
   	strcpy(MimeTypes[atomic_read(&MimeCount)].type,Type);
   	
   	atomic_inc(&MimeCount);
}


__inline char *ResolveMimeType(char *File)
/*

	The returned string is for READ ONLY!

*/
{	
	__u32	*I;
	int pos,lc,filelen;
	
	filelen = strlen(File);
	
	if (filelen<4) 
   	{	
   		return NULL;
   	}

	pos = filelen-4;
   	I=(__u32*)(File+pos);
   	
   	lc=0;
   	
   	while (lc<atomic_read(&MimeCount))
 	{
   		if (MimeTypes[lc].identifier == *I)
   	  		return MimeTypes[lc].type;
   	  	lc++;
   	} 	
   	return MimeTypes[0].type;
}

void HTTPHeader(char *Filename,int FileSize,time_t MTime, char *BUF)
{
	char TimeBuf[64];
     

#ifndef BENCHMARK     
		time_Unix2RFC(MTime,TimeBuf);
	sprintf(BUF,
	  "HTTP/1.0 200 OK\r\nServer: kHTTPd/0.1.3\r\nDate: %s\r\nContent-type: %s\r\nLast-modified: %s\r\nContent-length: %i\r\n\r\n",
	  CurrentTime,ResolveMimeType(Filename),TimeBuf,FileSize);
#else	
	sprintf(BUF,
	  "HTTP/1.0 200 OK\r\nContent-type: text/html\r\nContent-length: %i\r\n\r\n",
	  FileSize);
#endif
}



void ParseHeader(char *Buffer,int length, struct http_request *Head)
{
	char *Endval,*EOL,*tmp;
	
	Endval = Buffer + length;
	
/*	memset(Head->FileName,0,sizeof(Head->FileName));
	memset(Head->IMS,0,sizeof(Head->IMS));
	memset(Head->Agent,0,sizeof(Head->Agent));
	memset(Head->Host,0,sizeof(Head->Host));
*/	
	
	while (Buffer<Endval)
	{
		if (isspace(Buffer[0]))
		{
			Buffer++;
			continue;
		};
			
		
		EOL=strchr(Buffer,'\n');
		
		if (EOL==NULL) EOL=Endval;
		
		if (EOL-Buffer<4) 
		{
			Buffer++;
			continue;
		};
		
		if (strncmp("GET ",Buffer,4)==0)
		{
			int PrefixLen;
			Buffer+=4;
			
			tmp=strchr(Buffer,' ');
			if (tmp==0) tmp=EOL-1;
			
			if (tmp>Endval) continue;
			
			strcpy(Head->FileName,KHTTPD_PREFIX);
			PrefixLen = strlen(KHTTPD_PREFIX);
			Head->FileNameLength = min(255,tmp-Buffer+PrefixLen);		
			
			strncat(Head->FileName,Buffer,min(255-PrefixLen,tmp-Buffer));
					
			Buffer=EOL+1;	
#ifdef BENCHMARK
			break;
#endif						
			continue;
		}
#ifndef BENCHMARK		
		if (strncmp("If-Modified-Since: ",Buffer,19)==0)
		{
			Buffer+=19;
			
			strncpy(Head->IMS,Buffer,min(128,EOL-Buffer-1));
					
			Buffer=EOL+1;	
			continue;
		}

		if (strncmp("User-Agent: ",Buffer,12)==0)
		{
			Buffer+=12;
			
			strncpy(Head->Agent,Buffer,min(128,EOL-Buffer-1));
					
			Buffer=EOL+1;	
			continue;
		}
		

		if (strncmp("Host: ",Buffer,6)==0)
		{
			Buffer+=6;
			
			strncpy(Head->Host,Buffer,min(128,EOL-Buffer-1));
					
			Buffer=EOL+1;	
			continue;
		}
#endif		
		Buffer = EOL+1;  /* Skip line */
	}

}